import { T, pipe, cond, not, always, is, test } from 'ramda'
import decodePath from './decodePath'
import pathExp from './pathExp'

/**
 * Tests if a given string is a valid JSON Pointer
 * according to https://tools.ietf.org/html/rfc6901
 *
 * ** Sig: String -> Boolean|String **
 *
 * @function
 * @param {string} path - the path that should validate as a JSON Pointer
 * @returns {true|string} result - ** true ** if the path is a valid JSON Pointer or a ** string ** with the encountered error
 * @example
 *
 * isPath('/foo/bar') // true
 * isPath('foo!') // The path "foo!" did not match the encoded path regexp [regular expression]
 * isPath(123) // The path must be a string
 *
 */
const isPath = cond([
  [
    pipe(is(String), not),
    always('The path must be a string')
  ],
  [
    pipe(test(pathExp), not),
    x => `The path "${x}" did not match the encoded path regexp: ${pathExp}`
  ],
  [
    pipe(decodePath, test(pathExp), not),
    x => `The path "${x}" did not match, after decoding, the decoded path regexp: ${pathExp}`
  ],
  [T, T]
])

export default isPath
